@@ -1244,8 +1244,8 @@ EE flatten_bool_op(EnvI& env, Ctx& ctx, const Ctx& ctx0, const Ctx& ctx1, Expres
   Expression* le1 = nullptr;
 
   if (bot == BOT_IN) {
-    if (Expression::type(e1.r()).isPar() && Expression::type(e0.r()).isint() &&
-        !Expression::type(e0.r()).isOpt()) {
+    if (Expression::type(e0.r()).isint() && !Expression::type(e0.r()).isOpt() &&
+        Expression::type(e1.r()).isPar() && Expression::type(e1.r()).isSet()) {
       bool has_infinity = false;
       {
         GCLock lock;
@@ -1289,7 +1289,8 @@ EE flatten_bool_op(EnvI& env, Ctx& ctx, const Ctx& ctx0, const Ctx& ctx1, Expres
                                      callid);
     }
   } else if (isBuiltin && bot == BOT_EQ && ctx.b == C_ROOT && r == env.constants.varTrue &&
-             Expression::type(e1.r()).isPar() && Expression::isa<Id>(e0.r())) {
+             Expression::isa<Id>(e0.r()) && Expression::type(e1.r()).isPar() &&
+             !Expression::type(e1.r()).istuple() && !Expression::type(e1.r()).isrecord()) {
     // Par assignment to Id with no right hand side
     auto* vd = Expression::cast<Id>(e0.r())->decl();
     set_computed_value(env, vd, e1.r());
@@ -1953,7 +1954,7 @@ EE flatten_binop(EnvI& env, const Ctx& input_ctx, Expression* e, VarDecl* r, Var
       }
       Expression::type(alret, Expression::type(e));
       ret.b = conj(env, b, Ctx(), ee);
-      ret.r = bind(env, ctx, r, alret);
+      ret.r = bind(env, Ctx(), r, alret);
     } break;
   }
   return ret;
